<?php

declare(strict_types = 1);

namespace Grifart\ClassScaffolder;

use Grifart\ClassScaffolder\Definition\ClassDefinition;
use Grifart\ClassScaffolder\Definition\Types\CollectionType;


final class KotlinClassGenerator
{

    private static $typeMapping = [
        "string" => "String",
        "int" => "Int",
        "bool" => "Boolean",
        "float" => "Double"
    ];

    private function resolveKotlinType(Definition\Types\Type $type): string {
        $typeHint = $type->getTypeHint();

        if (isset(self::$typeMapping[$typeHint])) {
            $kotlinType = self::$typeMapping[$typeHint];
        } elseif ($type instanceof CollectionType) {
            $kotlinType = sprintf("Array<%s>", $this->resolveKotlinType($type->getElementType()));
        } else {
            $kotlinType = $typeHint;
        }

        if ($type->isNullable()) {
            $kotlinType .= "?";
        }

        return $kotlinType;
    }

    private function separateFields(): string {
        return sprintf(", ");
    }

	public function generateClass(ClassDefinition $definition): string
	{
		$class = sprintf("data class %s(", $definition->getClassName());

        $isFirstField = true;
		foreach ($definition->getFields() as $fieldName => $type) {
		    if ($isFirstField) {
                $isFirstField = false;
            } else {
                $class .= $this->separateFields();
            }

		    $field = sprintf("val %s: %s", (string) $fieldName, $this->resolveKotlinType($type));
		    $class .= $field;
        }

		$class .= ")";

		return $class;
	}
}
