<?php declare(strict_types=1);
/**
 * @testCase
 */

use Grifart\ClassScaffolder\Definition\ClassDefinition;
use \Grifart\ClassScaffolder\Definition\Types;

require_once __DIR__ . '/bootstrap.php';


class MyClassType {}

class MySecondClassType {}

(new class extends \Tester\TestCase {

	/** @var \Grifart\ClassScaffolder\KotlinClassGenerator */
	private $generator;

	protected function setUp()
	{
		parent::setUp();
		$this->generator = new \Grifart\ClassScaffolder\KotlinClassGenerator();
	}

	public function dataProvider_generator():array {
		return [
			[
				'data class CLS(val field: String)',
				new ClassDefinition('NS', 'CLS', [], ['field'=>Types\resolve('string')], [])
			],
            [
                'data class CLS(val field: String?)',
                new ClassDefinition('NS', 'CLS', [], ['field'=>Types\nullable(Types\resolve('string'))], [])
            ],
            [
                'data class CLS(val field: String, val field2: Int, val field3: Double)',
                new ClassDefinition('NS', 'CLS', [], ['field'=>Types\resolve('string'), 'field2'=>Types\resolve('int'), 'field3'=>Types\resolve('float')], [])
            ],
            [
                'data class CLS(val field: String, val field2: MyClassType, val field3: MySecondClassType?)',
                new ClassDefinition('NS', 'CLS', [], ['field'=>Types\resolve('string'), 'field2'=>Types\resolve(MyClassType::class), 'field3'=>Types\nullable(Types\resolve(MySecondClassType::class))], [])
            ],
            [
                'data class CLS(val field: Array<String>)',
                new ClassDefinition('NS', 'CLS', [], ['field'=>Types\listOf(Types\resolve('string'))], [])
            ],
            [
                'data class CLS(val field: Array<MyClassType>)',
                new ClassDefinition('NS', 'CLS', [], ['field'=>Types\listOf(Types\resolve(MyClassType::class))], [])
            ],

		];
	}

	/** @dataProvider dataProvider_generator */
	public function testGenerator(string $assertionString, ClassDefinition $definition) {
		\Tester\Assert::match(
			$assertionString,
			(string) $this->generator->generateClass($definition)
		);
	}


})->run();
